from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.request_for_input_event_provided_data_type_0 import RequestForInputEventProvidedDataType0
    from ..models.request_for_input_event_requested_data import RequestForInputEventRequestedData


T = TypeVar("T", bound="RequestForInputEvent")


@_attrs_define
class RequestForInputEvent:
    """
    Attributes:
        sequence (int | Unset):
        requested_data (RequestForInputEventRequestedData | Unset):
        provided_data (None | RequestForInputEventProvidedDataType0 | Unset):
        attribution (None | str | Unset):
        created_at (datetime.datetime | Unset):
        type_ (str | Unset):
    """

    sequence: int | Unset = UNSET
    requested_data: RequestForInputEventRequestedData | Unset = UNSET
    provided_data: None | RequestForInputEventProvidedDataType0 | Unset = UNSET
    attribution: None | str | Unset = UNSET
    created_at: datetime.datetime | Unset = UNSET
    type_: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.request_for_input_event_provided_data_type_0 import RequestForInputEventProvidedDataType0

        sequence = self.sequence

        requested_data: dict[str, Any] | Unset = UNSET
        if not isinstance(self.requested_data, Unset):
            requested_data = self.requested_data.to_dict()

        provided_data: dict[str, Any] | None | Unset
        if isinstance(self.provided_data, Unset):
            provided_data = UNSET
        elif isinstance(self.provided_data, RequestForInputEventProvidedDataType0):
            provided_data = self.provided_data.to_dict()
        else:
            provided_data = self.provided_data

        attribution: None | str | Unset
        if isinstance(self.attribution, Unset):
            attribution = UNSET
        else:
            attribution = self.attribution

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        type_ = self.type_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if sequence is not UNSET:
            field_dict["sequence"] = sequence
        if requested_data is not UNSET:
            field_dict["requestedData"] = requested_data
        if provided_data is not UNSET:
            field_dict["providedData"] = provided_data
        if attribution is not UNSET:
            field_dict["attribution"] = attribution
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.request_for_input_event_provided_data_type_0 import RequestForInputEventProvidedDataType0
        from ..models.request_for_input_event_requested_data import RequestForInputEventRequestedData

        d = dict(src_dict)
        sequence = d.pop("sequence", UNSET)

        _requested_data = d.pop("requestedData", UNSET)
        requested_data: RequestForInputEventRequestedData | Unset
        if isinstance(_requested_data, Unset):
            requested_data = UNSET
        else:
            requested_data = RequestForInputEventRequestedData.from_dict(_requested_data)

        def _parse_provided_data(data: object) -> None | RequestForInputEventProvidedDataType0 | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                provided_data_type_0 = RequestForInputEventProvidedDataType0.from_dict(data)

                return provided_data_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | RequestForInputEventProvidedDataType0 | Unset, data)

        provided_data = _parse_provided_data(d.pop("providedData", UNSET))

        def _parse_attribution(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        attribution = _parse_attribution(d.pop("attribution", UNSET))

        _created_at = d.pop("createdAt", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        type_ = d.pop("type", UNSET)

        request_for_input_event = cls(
            sequence=sequence,
            requested_data=requested_data,
            provided_data=provided_data,
            attribution=attribution,
            created_at=created_at,
            type_=type_,
        )

        request_for_input_event.additional_properties = d
        return request_for_input_event

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.info_event_1 import InfoEvent1
    from ..models.progress_event_1 import ProgressEvent1
    from ..models.user_input_event import UserInputEvent
    from ..models.user_request_for_input_event import UserRequestForInputEvent
    from ..models.worker_v2_next_page_token import WorkerV2NextPageToken


T = TypeVar("T", bound="GetWorkerEventsV2Response")


@_attrs_define
class GetWorkerEventsV2Response:
    """
    Attributes:
        events (list[InfoEvent1 | ProgressEvent1 | UserInputEvent | UserRequestForInputEvent] | Unset):
        next_page_token (WorkerV2NextPageToken | Unset):
    """

    events: list[InfoEvent1 | ProgressEvent1 | UserInputEvent | UserRequestForInputEvent] | Unset = UNSET
    next_page_token: WorkerV2NextPageToken | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.info_event_1 import InfoEvent1
        from ..models.progress_event_1 import ProgressEvent1
        from ..models.user_request_for_input_event import UserRequestForInputEvent

        events: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.events, Unset):
            events = []
            for events_item_data in self.events:
                events_item: dict[str, Any]
                if isinstance(events_item_data, InfoEvent1):
                    events_item = events_item_data.to_dict()
                elif isinstance(events_item_data, ProgressEvent1):
                    events_item = events_item_data.to_dict()
                elif isinstance(events_item_data, UserRequestForInputEvent):
                    events_item = events_item_data.to_dict()
                else:
                    events_item = events_item_data.to_dict()

                events.append(events_item)

        next_page_token: dict[str, Any] | Unset = UNSET
        if not isinstance(self.next_page_token, Unset):
            next_page_token = self.next_page_token.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if events is not UNSET:
            field_dict["events"] = events
        if next_page_token is not UNSET:
            field_dict["nextPageToken"] = next_page_token

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.info_event_1 import InfoEvent1
        from ..models.progress_event_1 import ProgressEvent1
        from ..models.user_input_event import UserInputEvent
        from ..models.user_request_for_input_event import UserRequestForInputEvent
        from ..models.worker_v2_next_page_token import WorkerV2NextPageToken

        d = dict(src_dict)
        _events = d.pop("events", UNSET)
        events: list[InfoEvent1 | ProgressEvent1 | UserInputEvent | UserRequestForInputEvent] | Unset = UNSET
        if _events is not UNSET:
            events = []
            for events_item_data in _events:

                def _parse_events_item(
                    data: object,
                ) -> InfoEvent1 | ProgressEvent1 | UserInputEvent | UserRequestForInputEvent:
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_worker_event_v2_type_0 = InfoEvent1.from_dict(data)

                        return componentsschemas_worker_event_v2_type_0
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_worker_event_v2_type_1 = ProgressEvent1.from_dict(data)

                        return componentsschemas_worker_event_v2_type_1
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_worker_event_v2_type_2 = UserRequestForInputEvent.from_dict(data)

                        return componentsschemas_worker_event_v2_type_2
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_worker_event_v2_type_3 = UserInputEvent.from_dict(data)

                    return componentsschemas_worker_event_v2_type_3

                events_item = _parse_events_item(events_item_data)

                events.append(events_item)

        _next_page_token = d.pop("nextPageToken", UNSET)
        next_page_token: WorkerV2NextPageToken | Unset
        if isinstance(_next_page_token, Unset):
            next_page_token = UNSET
        else:
            next_page_token = WorkerV2NextPageToken.from_dict(_next_page_token)

        get_worker_events_v2_response = cls(
            events=events,
            next_page_token=next_page_token,
        )

        get_worker_events_v2_response.additional_properties = d
        return get_worker_events_v2_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.worker_run_event_rating_secondary_id import WorkerRunEventRatingSecondaryId


T = TypeVar("T", bound="RatingDtoWorkerRunEventRatingSecondaryId")


@_attrs_define
class RatingDtoWorkerRunEventRatingSecondaryId:
    """
    Attributes:
        id (UUID | Unset):
        grade (float | Unset):
        written_feedback (str | Unset):
        secondary_id (WorkerRunEventRatingSecondaryId | Unset):
    """

    id: UUID | Unset = UNSET
    grade: float | Unset = UNSET
    written_feedback: str | Unset = UNSET
    secondary_id: WorkerRunEventRatingSecondaryId | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id: str | Unset = UNSET
        if not isinstance(self.id, Unset):
            id = str(self.id)

        grade = self.grade

        written_feedback = self.written_feedback

        secondary_id: dict[str, Any] | Unset = UNSET
        if not isinstance(self.secondary_id, Unset):
            secondary_id = self.secondary_id.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if grade is not UNSET:
            field_dict["grade"] = grade
        if written_feedback is not UNSET:
            field_dict["writtenFeedback"] = written_feedback
        if secondary_id is not UNSET:
            field_dict["secondaryId"] = secondary_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.worker_run_event_rating_secondary_id import WorkerRunEventRatingSecondaryId

        d = dict(src_dict)
        _id = d.pop("id", UNSET)
        id: UUID | Unset
        if isinstance(_id, Unset):
            id = UNSET
        else:
            id = UUID(_id)

        grade = d.pop("grade", UNSET)

        written_feedback = d.pop("writtenFeedback", UNSET)

        _secondary_id = d.pop("secondaryId", UNSET)
        secondary_id: WorkerRunEventRatingSecondaryId | Unset
        if isinstance(_secondary_id, Unset):
            secondary_id = UNSET
        else:
            secondary_id = WorkerRunEventRatingSecondaryId.from_dict(_secondary_id)

        rating_dto_worker_run_event_rating_secondary_id = cls(
            id=id,
            grade=grade,
            written_feedback=written_feedback,
            secondary_id=secondary_id,
        )

        rating_dto_worker_run_event_rating_secondary_id.additional_properties = d
        return rating_dto_worker_run_event_rating_secondary_id

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="UpdateTagRequest")


@_attrs_define
class UpdateTagRequest:
    """
    Attributes:
        new_name (None | str | Unset):
        new_description (None | str | Unset):
    """

    new_name: None | str | Unset = UNSET
    new_description: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        new_name: None | str | Unset
        if isinstance(self.new_name, Unset):
            new_name = UNSET
        else:
            new_name = self.new_name

        new_description: None | str | Unset
        if isinstance(self.new_description, Unset):
            new_description = UNSET
        else:
            new_description = self.new_description

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if new_name is not UNSET:
            field_dict["newName"] = new_name
        if new_description is not UNSET:
            field_dict["newDescription"] = new_description

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_new_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        new_name = _parse_new_name(d.pop("newName", UNSET))

        def _parse_new_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        new_description = _parse_new_description(d.pop("newDescription", UNSET))

        update_tag_request = cls(
            new_name=new_name,
            new_description=new_description,
        )

        update_tag_request.additional_properties = d
        return update_tag_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any
from urllib.parse import quote

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.managed_file_system_api_dto import ManagedFileSystemApiDto
from ...models.remote_file_system_api_dto import RemoteFileSystemApiDto
from ...types import Response


def _get_kwargs(
    file_system_rid: str,
) -> dict[str, Any]:

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/v1/file-systems/{file_system_rid}".format(
            file_system_rid=quote(str(file_system_rid), safe=""),
        ),
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> ManagedFileSystemApiDto | RemoteFileSystemApiDto | None:
    if response.status_code == 200:

        def _parse_response_200(data: object) -> ManagedFileSystemApiDto | RemoteFileSystemApiDto:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_file_system_api_dto_type_0 = RemoteFileSystemApiDto.from_dict(data)

                return componentsschemas_file_system_api_dto_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_file_system_api_dto_type_1 = ManagedFileSystemApiDto.from_dict(data)

            return componentsschemas_file_system_api_dto_type_1

        response_200 = _parse_response_200(response.json())

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[ManagedFileSystemApiDto | RemoteFileSystemApiDto]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    file_system_rid: str,
    *,
    client: AuthenticatedClient | Client,
) -> Response[ManagedFileSystemApiDto | RemoteFileSystemApiDto]:
    """Get File System

    Args:
        file_system_rid (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ManagedFileSystemApiDto | RemoteFileSystemApiDto]
    """

    kwargs = _get_kwargs(
        file_system_rid=file_system_rid,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    file_system_rid: str,
    *,
    client: AuthenticatedClient | Client,
) -> ManagedFileSystemApiDto | RemoteFileSystemApiDto | None:
    """Get File System

    Args:
        file_system_rid (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ManagedFileSystemApiDto | RemoteFileSystemApiDto
    """

    return sync_detailed(
        file_system_rid=file_system_rid,
        client=client,
    ).parsed


async def asyncio_detailed(
    file_system_rid: str,
    *,
    client: AuthenticatedClient | Client,
) -> Response[ManagedFileSystemApiDto | RemoteFileSystemApiDto]:
    """Get File System

    Args:
        file_system_rid (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ManagedFileSystemApiDto | RemoteFileSystemApiDto]
    """

    kwargs = _get_kwargs(
        file_system_rid=file_system_rid,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    file_system_rid: str,
    *,
    client: AuthenticatedClient | Client,
) -> ManagedFileSystemApiDto | RemoteFileSystemApiDto | None:
    """Get File System

    Args:
        file_system_rid (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ManagedFileSystemApiDto | RemoteFileSystemApiDto
    """

    return (
        await asyncio_detailed(
            file_system_rid=file_system_rid,
            client=client,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.worker_metadata_dto_input_schema import WorkerMetadataDtoInputSchema
    from ..models.worker_metadata_dto_response_schema import WorkerMetadataDtoResponseSchema


T = TypeVar("T", bound="WorkerMetadataDto")


@_attrs_define
class WorkerMetadataDto:
    """
    Attributes:
        version (str | Unset):
        description (str | Unset):
        input_schema (WorkerMetadataDtoInputSchema | Unset):
        response_schema (WorkerMetadataDtoResponseSchema | Unset):
    """

    version: str | Unset = UNSET
    description: str | Unset = UNSET
    input_schema: WorkerMetadataDtoInputSchema | Unset = UNSET
    response_schema: WorkerMetadataDtoResponseSchema | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        version = self.version

        description = self.description

        input_schema: dict[str, Any] | Unset = UNSET
        if not isinstance(self.input_schema, Unset):
            input_schema = self.input_schema.to_dict()

        response_schema: dict[str, Any] | Unset = UNSET
        if not isinstance(self.response_schema, Unset):
            response_schema = self.response_schema.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if version is not UNSET:
            field_dict["version"] = version
        if description is not UNSET:
            field_dict["description"] = description
        if input_schema is not UNSET:
            field_dict["inputSchema"] = input_schema
        if response_schema is not UNSET:
            field_dict["responseSchema"] = response_schema

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.worker_metadata_dto_input_schema import WorkerMetadataDtoInputSchema
        from ..models.worker_metadata_dto_response_schema import WorkerMetadataDtoResponseSchema

        d = dict(src_dict)
        version = d.pop("version", UNSET)

        description = d.pop("description", UNSET)

        _input_schema = d.pop("inputSchema", UNSET)
        input_schema: WorkerMetadataDtoInputSchema | Unset
        if isinstance(_input_schema, Unset):
            input_schema = UNSET
        else:
            input_schema = WorkerMetadataDtoInputSchema.from_dict(_input_schema)

        _response_schema = d.pop("responseSchema", UNSET)
        response_schema: WorkerMetadataDtoResponseSchema | Unset
        if isinstance(_response_schema, Unset):
            response_schema = UNSET
        else:
            response_schema = WorkerMetadataDtoResponseSchema.from_dict(_response_schema)

        worker_metadata_dto = cls(
            version=version,
            description=description,
            input_schema=input_schema,
            response_schema=response_schema,
        )

        worker_metadata_dto.additional_properties = d
        return worker_metadata_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any
from urllib.parse import quote

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.workflow_event import WorkflowEvent
from ...types import UNSET, Response, Unset


def _get_kwargs(
    workflow_id: str,
    run_rid: str,
    *,
    last_event_id_query: str | Unset = UNSET,
    last_event_id_header: str | Unset = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}
    if not isinstance(last_event_id_header, Unset):
        headers["Last-Event-ID"] = last_event_id_header

    params: dict[str, Any] = {}

    params["lastEventId"] = last_event_id_query

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/v1/workflows/{workflow_id}/runs/{run_rid}/stream".format(
            workflow_id=quote(str(workflow_id), safe=""),
            run_rid=quote(str(run_rid), safe=""),
        ),
        "params": params,
    }

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> list[WorkflowEvent] | None:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.text
        for response_200_item_data in _response_200:
            response_200_item = WorkflowEvent.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[list[WorkflowEvent]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    workflow_id: str,
    run_rid: str,
    *,
    client: AuthenticatedClient | Client,
    last_event_id_query: str | Unset = UNSET,
    last_event_id_header: str | Unset = UNSET,
) -> Response[list[WorkflowEvent]]:
    """Stream Events

    Args:
        workflow_id (str):
        run_rid (str):
        last_event_id_query (str | Unset):
        last_event_id_header (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[list[WorkflowEvent]]
    """

    kwargs = _get_kwargs(
        workflow_id=workflow_id,
        run_rid=run_rid,
        last_event_id_query=last_event_id_query,
        last_event_id_header=last_event_id_header,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    workflow_id: str,
    run_rid: str,
    *,
    client: AuthenticatedClient | Client,
    last_event_id_query: str | Unset = UNSET,
    last_event_id_header: str | Unset = UNSET,
) -> list[WorkflowEvent] | None:
    """Stream Events

    Args:
        workflow_id (str):
        run_rid (str):
        last_event_id_query (str | Unset):
        last_event_id_header (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        list[WorkflowEvent]
    """

    return sync_detailed(
        workflow_id=workflow_id,
        run_rid=run_rid,
        client=client,
        last_event_id_query=last_event_id_query,
        last_event_id_header=last_event_id_header,
    ).parsed


async def asyncio_detailed(
    workflow_id: str,
    run_rid: str,
    *,
    client: AuthenticatedClient | Client,
    last_event_id_query: str | Unset = UNSET,
    last_event_id_header: str | Unset = UNSET,
) -> Response[list[WorkflowEvent]]:
    """Stream Events

    Args:
        workflow_id (str):
        run_rid (str):
        last_event_id_query (str | Unset):
        last_event_id_header (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[list[WorkflowEvent]]
    """

    kwargs = _get_kwargs(
        workflow_id=workflow_id,
        run_rid=run_rid,
        last_event_id_query=last_event_id_query,
        last_event_id_header=last_event_id_header,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    workflow_id: str,
    run_rid: str,
    *,
    client: AuthenticatedClient | Client,
    last_event_id_query: str | Unset = UNSET,
    last_event_id_header: str | Unset = UNSET,
) -> list[WorkflowEvent] | None:
    """Stream Events

    Args:
        workflow_id (str):
        run_rid (str):
        last_event_id_query (str | Unset):
        last_event_id_header (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        list[WorkflowEvent]
    """

    return (
        await asyncio_detailed(
            workflow_id=workflow_id,
            run_rid=run_rid,
            client=client,
            last_event_id_query=last_event_id_query,
            last_event_id_header=last_event_id_header,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.cron_trigger import CronTrigger
    from ..models.interval_trigger import IntervalTrigger
    from ..models.on_upload_trigger import OnUploadTrigger
    from ..models.update_indexing_schedule_request_indexing_settings_type_0 import (
        UpdateIndexingScheduleRequestIndexingSettingsType0,
    )


T = TypeVar("T", bound="UpdateIndexingScheduleRequest")


@_attrs_define
class UpdateIndexingScheduleRequest:
    """
    Attributes:
        name (None | str | Unset): Updated display name
        description (None | str | Unset): Updated description
        target_minimum_version (int | None | Unset): Updated minimum indexer version
        indexing_settings (None | Unset | UpdateIndexingScheduleRequestIndexingSettingsType0): Updated indexer-specific
            settings
        trigger (CronTrigger | IntervalTrigger | None | OnUploadTrigger | Unset): Updated trigger configuration
        enabled (bool | None | Unset): Updated enabled status
    """

    name: None | str | Unset = UNSET
    description: None | str | Unset = UNSET
    target_minimum_version: int | None | Unset = UNSET
    indexing_settings: None | Unset | UpdateIndexingScheduleRequestIndexingSettingsType0 = UNSET
    trigger: CronTrigger | IntervalTrigger | None | OnUploadTrigger | Unset = UNSET
    enabled: bool | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.cron_trigger import CronTrigger
        from ..models.interval_trigger import IntervalTrigger
        from ..models.on_upload_trigger import OnUploadTrigger
        from ..models.update_indexing_schedule_request_indexing_settings_type_0 import (
            UpdateIndexingScheduleRequestIndexingSettingsType0,
        )

        name: None | str | Unset
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        target_minimum_version: int | None | Unset
        if isinstance(self.target_minimum_version, Unset):
            target_minimum_version = UNSET
        else:
            target_minimum_version = self.target_minimum_version

        indexing_settings: dict[str, Any] | None | Unset
        if isinstance(self.indexing_settings, Unset):
            indexing_settings = UNSET
        elif isinstance(self.indexing_settings, UpdateIndexingScheduleRequestIndexingSettingsType0):
            indexing_settings = self.indexing_settings.to_dict()
        else:
            indexing_settings = self.indexing_settings

        trigger: dict[str, Any] | None | Unset
        if isinstance(self.trigger, Unset):
            trigger = UNSET
        elif isinstance(self.trigger, OnUploadTrigger):
            trigger = self.trigger.to_dict()
        elif isinstance(self.trigger, CronTrigger):
            trigger = self.trigger.to_dict()
        elif isinstance(self.trigger, IntervalTrigger):
            trigger = self.trigger.to_dict()
        else:
            trigger = self.trigger

        enabled: bool | None | Unset
        if isinstance(self.enabled, Unset):
            enabled = UNSET
        else:
            enabled = self.enabled

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if target_minimum_version is not UNSET:
            field_dict["targetMinimumVersion"] = target_minimum_version
        if indexing_settings is not UNSET:
            field_dict["indexingSettings"] = indexing_settings
        if trigger is not UNSET:
            field_dict["trigger"] = trigger
        if enabled is not UNSET:
            field_dict["enabled"] = enabled

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.cron_trigger import CronTrigger
        from ..models.interval_trigger import IntervalTrigger
        from ..models.on_upload_trigger import OnUploadTrigger
        from ..models.update_indexing_schedule_request_indexing_settings_type_0 import (
            UpdateIndexingScheduleRequestIndexingSettingsType0,
        )

        d = dict(src_dict)

        def _parse_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_target_minimum_version(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        target_minimum_version = _parse_target_minimum_version(d.pop("targetMinimumVersion", UNSET))

        def _parse_indexing_settings(data: object) -> None | Unset | UpdateIndexingScheduleRequestIndexingSettingsType0:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                indexing_settings_type_0 = UpdateIndexingScheduleRequestIndexingSettingsType0.from_dict(data)

                return indexing_settings_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UpdateIndexingScheduleRequestIndexingSettingsType0, data)

        indexing_settings = _parse_indexing_settings(d.pop("indexingSettings", UNSET))

        def _parse_trigger(data: object) -> CronTrigger | IntervalTrigger | None | OnUploadTrigger | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_schedule_trigger_type_0 = OnUploadTrigger.from_dict(data)

                return componentsschemas_schedule_trigger_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_schedule_trigger_type_1 = CronTrigger.from_dict(data)

                return componentsschemas_schedule_trigger_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_schedule_trigger_type_2 = IntervalTrigger.from_dict(data)

                return componentsschemas_schedule_trigger_type_2
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(CronTrigger | IntervalTrigger | None | OnUploadTrigger | Unset, data)

        trigger = _parse_trigger(d.pop("trigger", UNSET))

        def _parse_enabled(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        enabled = _parse_enabled(d.pop("enabled", UNSET))

        update_indexing_schedule_request = cls(
            name=name,
            description=description,
            target_minimum_version=target_minimum_version,
            indexing_settings=indexing_settings,
            trigger=trigger,
            enabled=enabled,
        )

        update_indexing_schedule_request.additional_properties = d
        return update_indexing_schedule_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="SchemaLocation")


@_attrs_define
class SchemaLocation:
    """
    Attributes:
        root_document_uri (str | Unset):
        pointer_to_location (list[str] | Unset):
    """

    root_document_uri: str | Unset = UNSET
    pointer_to_location: list[str] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        root_document_uri = self.root_document_uri

        pointer_to_location: list[str] | Unset = UNSET
        if not isinstance(self.pointer_to_location, Unset):
            pointer_to_location = self.pointer_to_location

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if root_document_uri is not UNSET:
            field_dict["rootDocumentURI"] = root_document_uri
        if pointer_to_location is not UNSET:
            field_dict["pointerToLocation"] = pointer_to_location

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        root_document_uri = d.pop("rootDocumentURI", UNSET)

        pointer_to_location = cast(list[str], d.pop("pointerToLocation", UNSET))

        schema_location = cls(
            root_document_uri=root_document_uri,
            pointer_to_location=pointer_to_location,
        )

        schema_location.additional_properties = d
        return schema_location

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains some shared types for properties"""

from collections.abc import Mapping, MutableMapping
from http import HTTPStatus
from typing import IO, BinaryIO, Generic, Literal, TypeVar

from attrs import define


class Unset:
    def __bool__(self) -> Literal[False]:
        return False


UNSET: Unset = Unset()

# The types that `httpx.Client(files=)` can accept, copied from that library.
FileContent = IO[bytes] | bytes | str
FileTypes = (
    # (filename, file (or bytes), content_type)
    tuple[str | None, FileContent, str | None]
    # (filename, file (or bytes), content_type, headers)
    | tuple[str | None, FileContent, str | None, Mapping[str, str]]
)
RequestFiles = list[tuple[str, FileTypes]]


@define
class File:
    """Contains information for file uploads"""

    payload: BinaryIO
    file_name: str | None = None
    mime_type: str | None = None

    def to_tuple(self) -> FileTypes:
        """Return a tuple representation that httpx will accept for multipart/form-data"""
        return self.file_name, self.payload, self.mime_type


T = TypeVar("T")


@define
class Response(Generic[T]):
    """A response from an endpoint"""

    status_code: HTTPStatus
    content: bytes
    headers: MutableMapping[str, str]
    parsed: T | None


__all__ = ["UNSET", "File", "FileTypes", "RequestFiles", "Response", "Unset"]



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="S3ConnectionSettingsApiDto")


@_attrs_define
class S3ConnectionSettingsApiDto:
    """
    Attributes:
        region (str):
        access_key (str):
        secret_key (str):
        url (str):
        type_ (str | Unset):
    """

    region: str
    access_key: str
    secret_key: str
    url: str
    type_: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        region = self.region

        access_key = self.access_key

        secret_key = self.secret_key

        url = self.url

        type_ = self.type_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "region": region,
                "accessKey": access_key,
                "secretKey": secret_key,
                "url": url,
            }
        )
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        region = d.pop("region")

        access_key = d.pop("accessKey")

        secret_key = d.pop("secretKey")

        url = d.pop("url")

        type_ = d.pop("type", UNSET)

        s3_connection_settings_api_dto = cls(
            region=region,
            access_key=access_key,
            secret_key=secret_key,
            url=url,
            type_=type_,
        )

        s3_connection_settings_api_dto.additional_properties = d
        return s3_connection_settings_api_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class JsonNodeType(str, Enum):
    ARRAY = "ARRAY"
    BINARY = "BINARY"
    BOOLEAN = "BOOLEAN"
    MISSING = "MISSING"
    NULL = "NULL"
    NUMBER = "NUMBER"
    OBJECT = "OBJECT"
    POJO = "POJO"
    STRING = "STRING"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.biotech_analysis_image_annotated_citation_api_dto import BiotechAnalysisImageAnnotatedCitationApiDto
    from ..models.highlighted_text_citation_metadata_api_dto import HighlightedTextCitationMetadataApiDto
    from ..models.web_page_citation_metadata_api_dto import WebPageCitationMetadataApiDto


T = TypeVar("T", bound="CitationApiDto")


@_attrs_define
class CitationApiDto:
    """
    Attributes:
        id (str | Unset):
        input_uri (str | Unset):
        output_artifact_uri (None | str | Unset):
        metadata (BiotechAnalysisImageAnnotatedCitationApiDto | HighlightedTextCitationMetadataApiDto | Unset |
            WebPageCitationMetadataApiDto):
    """

    id: str | Unset = UNSET
    input_uri: str | Unset = UNSET
    output_artifact_uri: None | str | Unset = UNSET
    metadata: (
        BiotechAnalysisImageAnnotatedCitationApiDto
        | HighlightedTextCitationMetadataApiDto
        | Unset
        | WebPageCitationMetadataApiDto
    ) = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.highlighted_text_citation_metadata_api_dto import HighlightedTextCitationMetadataApiDto
        from ..models.web_page_citation_metadata_api_dto import WebPageCitationMetadataApiDto

        id = self.id

        input_uri = self.input_uri

        output_artifact_uri: None | str | Unset
        if isinstance(self.output_artifact_uri, Unset):
            output_artifact_uri = UNSET
        else:
            output_artifact_uri = self.output_artifact_uri

        metadata: dict[str, Any] | Unset
        if isinstance(self.metadata, Unset):
            metadata = UNSET
        elif isinstance(self.metadata, HighlightedTextCitationMetadataApiDto):
            metadata = self.metadata.to_dict()
        elif isinstance(self.metadata, WebPageCitationMetadataApiDto):
            metadata = self.metadata.to_dict()
        else:
            metadata = self.metadata.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if input_uri is not UNSET:
            field_dict["inputUri"] = input_uri
        if output_artifact_uri is not UNSET:
            field_dict["outputArtifactUri"] = output_artifact_uri
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.biotech_analysis_image_annotated_citation_api_dto import (
            BiotechAnalysisImageAnnotatedCitationApiDto,
        )
        from ..models.highlighted_text_citation_metadata_api_dto import HighlightedTextCitationMetadataApiDto
        from ..models.web_page_citation_metadata_api_dto import WebPageCitationMetadataApiDto

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        input_uri = d.pop("inputUri", UNSET)

        def _parse_output_artifact_uri(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        output_artifact_uri = _parse_output_artifact_uri(d.pop("outputArtifactUri", UNSET))

        def _parse_metadata(
            data: object,
        ) -> (
            BiotechAnalysisImageAnnotatedCitationApiDto
            | HighlightedTextCitationMetadataApiDto
            | Unset
            | WebPageCitationMetadataApiDto
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_citation_metadata_api_dto_type_0 = HighlightedTextCitationMetadataApiDto.from_dict(
                    data
                )

                return componentsschemas_citation_metadata_api_dto_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_citation_metadata_api_dto_type_1 = WebPageCitationMetadataApiDto.from_dict(data)

                return componentsschemas_citation_metadata_api_dto_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_citation_metadata_api_dto_type_2 = BiotechAnalysisImageAnnotatedCitationApiDto.from_dict(
                data
            )

            return componentsschemas_citation_metadata_api_dto_type_2

        metadata = _parse_metadata(d.pop("metadata", UNSET))

        citation_api_dto = cls(
            id=id,
            input_uri=input_uri,
            output_artifact_uri=output_artifact_uri,
            metadata=metadata,
        )

        citation_api_dto.additional_properties = d
        return citation_api_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.execution_trigger_reason import ExecutionTriggerReason
from ..types import UNSET, Unset

T = TypeVar("T", bound="ScheduleExecutionDto")


@_attrs_define
class ScheduleExecutionDto:
    """
    Attributes:
        execution_rid (str | Unset):
        schedule_rid (str | Unset):
        task_rid (str | Unset):
        triggered_at (datetime.datetime | Unset):
        reason (ExecutionTriggerReason | Unset):
        trigger_details (None | str | Unset):
    """

    execution_rid: str | Unset = UNSET
    schedule_rid: str | Unset = UNSET
    task_rid: str | Unset = UNSET
    triggered_at: datetime.datetime | Unset = UNSET
    reason: ExecutionTriggerReason | Unset = UNSET
    trigger_details: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        execution_rid = self.execution_rid

        schedule_rid = self.schedule_rid

        task_rid = self.task_rid

        triggered_at: str | Unset = UNSET
        if not isinstance(self.triggered_at, Unset):
            triggered_at = self.triggered_at.isoformat()

        reason: str | Unset = UNSET
        if not isinstance(self.reason, Unset):
            reason = self.reason.value

        trigger_details: None | str | Unset
        if isinstance(self.trigger_details, Unset):
            trigger_details = UNSET
        else:
            trigger_details = self.trigger_details

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if execution_rid is not UNSET:
            field_dict["executionRid"] = execution_rid
        if schedule_rid is not UNSET:
            field_dict["scheduleRid"] = schedule_rid
        if task_rid is not UNSET:
            field_dict["taskRid"] = task_rid
        if triggered_at is not UNSET:
            field_dict["triggeredAt"] = triggered_at
        if reason is not UNSET:
            field_dict["reason"] = reason
        if trigger_details is not UNSET:
            field_dict["triggerDetails"] = trigger_details

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        execution_rid = d.pop("executionRid", UNSET)

        schedule_rid = d.pop("scheduleRid", UNSET)

        task_rid = d.pop("taskRid", UNSET)

        _triggered_at = d.pop("triggeredAt", UNSET)
        triggered_at: datetime.datetime | Unset
        if isinstance(_triggered_at, Unset):
            triggered_at = UNSET
        else:
            triggered_at = isoparse(_triggered_at)

        _reason = d.pop("reason", UNSET)
        reason: ExecutionTriggerReason | Unset
        if isinstance(_reason, Unset):
            reason = UNSET
        else:
            reason = ExecutionTriggerReason(_reason)

        def _parse_trigger_details(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        trigger_details = _parse_trigger_details(d.pop("triggerDetails", UNSET))

        schedule_execution_dto = cls(
            execution_rid=execution_rid,
            schedule_rid=schedule_rid,
            task_rid=task_rid,
            triggered_at=triggered_at,
            reason=reason,
            trigger_details=trigger_details,
        )

        schedule_execution_dto.additional_properties = d
        return schedule_execution_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="HighlightedTextCitationMetadataApiDto")


@_attrs_define
class HighlightedTextCitationMetadataApiDto:
    """
    Attributes:
        page_index (int | Unset):
        paragraph_index (int | Unset):
        text (str | Unset):
    """

    page_index: int | Unset = UNSET
    paragraph_index: int | Unset = UNSET
    text: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        page_index = self.page_index

        paragraph_index = self.paragraph_index

        text = self.text

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if page_index is not UNSET:
            field_dict["pageIndex"] = page_index
        if paragraph_index is not UNSET:
            field_dict["paragraphIndex"] = paragraph_index
        if text is not UNSET:
            field_dict["text"] = text

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        page_index = d.pop("pageIndex", UNSET)

        paragraph_index = d.pop("paragraphIndex", UNSET)

        text = d.pop("text", UNSET)

        highlighted_text_citation_metadata_api_dto = cls(
            page_index=page_index,
            paragraph_index=paragraph_index,
            text=text,
        )

        highlighted_text_citation_metadata_api_dto.additional_properties = d
        return highlighted_text_citation_metadata_api_dto

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.citation_api_dto import CitationApiDto


T = TypeVar("T", bound="BulkGetCitationsResponse")


@_attrs_define
class BulkGetCitationsResponse:
    """
    Attributes:
        citations (list[CitationApiDto] | Unset):
    """

    citations: list[CitationApiDto] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        citations: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.citations, Unset):
            citations = []
            for citations_item_data in self.citations:
                citations_item = citations_item_data.to_dict()
                citations.append(citations_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if citations is not UNSET:
            field_dict["citations"] = citations

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.citation_api_dto import CitationApiDto

        d = dict(src_dict)
        _citations = d.pop("citations", UNSET)
        citations: list[CitationApiDto] | Unset = UNSET
        if _citations is not UNSET:
            citations = []
            for citations_item_data in _citations:
                citations_item = CitationApiDto.from_dict(citations_item_data)

                citations.append(citations_item)

        bulk_get_citations_response = cls(
            citations=citations,
        )

        bulk_get_citations_response.additional_properties = d
        return bulk_get_citations_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.indexing_schedule_dto import IndexingScheduleDto


T = TypeVar("T", bound="UpdateIndexingScheduleResponse")


@_attrs_define
class UpdateIndexingScheduleResponse:
    """
    Attributes:
        schedule (IndexingScheduleDto | Unset):
    """

    schedule: IndexingScheduleDto | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        schedule: dict[str, Any] | Unset = UNSET
        if not isinstance(self.schedule, Unset):
            schedule = self.schedule.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if schedule is not UNSET:
            field_dict["schedule"] = schedule

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.indexing_schedule_dto import IndexingScheduleDto

        d = dict(src_dict)
        _schedule = d.pop("schedule", UNSET)
        schedule: IndexingScheduleDto | Unset
        if isinstance(_schedule, Unset):
            schedule = UNSET
        else:
            schedule = IndexingScheduleDto.from_dict(_schedule)

        update_indexing_schedule_response = cls(
            schedule=schedule,
        )

        update_indexing_schedule_response.additional_properties = d
        return update_indexing_schedule_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any
from urllib.parse import quote

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.paged_list_file_system_response import PagedListFileSystemResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    file_system_rid: str,
    *,
    folder: str | Unset = UNSET,
    page_size: int | Unset = 1000,
    resume_token: str | Unset = UNSET,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    params["folder"] = folder

    params["pageSize"] = page_size

    params["resumeToken"] = resume_token

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/v1/file-systems/{file_system_rid}/contents/files".format(
            file_system_rid=quote(str(file_system_rid), safe=""),
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> PagedListFileSystemResponse | None:
    if response.status_code == 200:
        response_200 = PagedListFileSystemResponse.from_dict(response.json())

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[PagedListFileSystemResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    file_system_rid: str,
    *,
    client: AuthenticatedClient | Client,
    folder: str | Unset = UNSET,
    page_size: int | Unset = 1000,
    resume_token: str | Unset = UNSET,
) -> Response[PagedListFileSystemResponse]:
    """List Files Flattened

    Args:
        file_system_rid (str):
        folder (str | Unset):
        page_size (int | Unset):  Default: 1000.
        resume_token (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PagedListFileSystemResponse]
    """

    kwargs = _get_kwargs(
        file_system_rid=file_system_rid,
        folder=folder,
        page_size=page_size,
        resume_token=resume_token,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    file_system_rid: str,
    *,
    client: AuthenticatedClient | Client,
    folder: str | Unset = UNSET,
    page_size: int | Unset = 1000,
    resume_token: str | Unset = UNSET,
) -> PagedListFileSystemResponse | None:
    """List Files Flattened

    Args:
        file_system_rid (str):
        folder (str | Unset):
        page_size (int | Unset):  Default: 1000.
        resume_token (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PagedListFileSystemResponse
    """

    return sync_detailed(
        file_system_rid=file_system_rid,
        client=client,
        folder=folder,
        page_size=page_size,
        resume_token=resume_token,
    ).parsed


async def asyncio_detailed(
    file_system_rid: str,
    *,
    client: AuthenticatedClient | Client,
    folder: str | Unset = UNSET,
    page_size: int | Unset = 1000,
    resume_token: str | Unset = UNSET,
) -> Response[PagedListFileSystemResponse]:
    """List Files Flattened

    Args:
        file_system_rid (str):
        folder (str | Unset):
        page_size (int | Unset):  Default: 1000.
        resume_token (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PagedListFileSystemResponse]
    """

    kwargs = _get_kwargs(
        file_system_rid=file_system_rid,
        folder=folder,
        page_size=page_size,
        resume_token=resume_token,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    file_system_rid: str,
    *,
    client: AuthenticatedClient | Client,
    folder: str | Unset = UNSET,
    page_size: int | Unset = 1000,
    resume_token: str | Unset = UNSET,
) -> PagedListFileSystemResponse | None:
    """List Files Flattened

    Args:
        file_system_rid (str):
        folder (str | Unset):
        page_size (int | Unset):  Default: 1000.
        resume_token (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PagedListFileSystemResponse
    """

    return (
        await asyncio_detailed(
            file_system_rid=file_system_rid,
            client=client,
            folder=folder,
            page_size=page_size,
            resume_token=resume_token,
        )
    ).parsed
